# DEFAULT VALUES
DEF_PORT = 7788
DEF_IP = '127.0.0.1'

# COMMON VARIABLES
MAX_CON = 5
MAX_PKG = 1024
ENCODING = 'utf-8'

# JIM VARIABLES
ACTION = 'action'
TIME = 'time'
USER = 'user'
ACCOUNT_NAME = 'account_name'
PRESENCE = 'presence'
RESPONSE = 'response'
ERROR = 'error'
SENDER = 'sender'
DESTINATION = 'destination'
MESSAGE = 'message'
MESSAGE_TEXT = 'message_text'
EXIT = 'exit'
GET_FRIENDS = 'get_friends'
LIST_INFO = 'data_list'
REMOVE_FRIEND = 'remove'
ADD_FRIEND = 'add'
USERS_REQUEST = 'get_users'
DATA = 'bin'
ERROR = 'error'
PUBLIC_KEY = 'pubkey'
PUBLIC_KEY_REQUEST = 'pubkey_need'

RESP_OK = {RESPONSE: 200}
RESP_202 = {RESPONSE: 202, LIST_INFO: None}
RESP_BAD = {RESPONSE: 400, ERROR: 'Bad Request'}
RESP_205 = {RESPONSE: 205}
RESP_511 = {RESPONSE: 511, DATA: None}



import sys
import logging
# sys.path.append('../')
from PyQt5.QtWidgets import QDialog, QLabel, QComboBox, QPushButton
from PyQt5.QtCore import Qt

LOG = logging.getLogger('client')


class AddContactDialog(QDialog):
    """
    Dialogue for adding a user to the friends list.
    Provides the user with a list of possible friends and
    adds the selected one to friends.
    """
    def __init__(self, transport, database):
        super().__init__()
        self.transport = transport
        self.database = database
        self.setFixedSize(350, 120)
        self.setWindowTitle('Select user to add:')
        self.setAttribute(Qt.WA_DeleteOnClose)
        self.setModal(True)
        self.selector_label = QLabel('Select user to add:', self)
        self.selector_label.setFixedSize(200, 20)
        self.selector_label.move(10, 0)
        self.selector = QComboBox(self)
        self.selector.setFixedSize(200, 20)
        self.selector.move(10, 30)
        self.btn_refresh = QPushButton('Refresh list', self)
        self.btn_refresh.setFixedSize(100, 30)
        self.btn_refresh.move(60, 60)
        self.btn_ok = QPushButton('Add', self)
        self.btn_ok.setFixedSize(100, 30)
        self.btn_ok.move(230, 20)
        self.btn_cancel = QPushButton('Cancel', self)
        self.btn_cancel.setFixedSize(100, 30)
        self.btn_cancel.move(230, 60)
        self.btn_cancel.clicked.connect(self.close)
        self.possible_friends_update()
        self.btn_refresh.clicked.connect(self.update_possible_friends)

    def possible_friends_update(self):
        """
        A method for filling out the list of possible friends.
        Creates a list of all registered users
        except for those already added to friends and yourself.
        """
        self.selector.clear()
        friends_list = set(self.database.get_friends())
        users_list = set(self.database.get_all_users())
        users_list.remove(self.transport.username)
        self.selector.addItems(users_list - friends_list)

    def update_possible_friends(self):
        """
        Method for updating the list of possible friends. Requests from the server
        the list of known users and wipes out the contents of the window.
        """
        try:
            self.transport.user_list_request()
        except OSError:
            pass
        else:
            LOG.debug('Refresh users complete.')
            self.possible_friends_update()



import sys
import logging
sys.path.append('../../../')
from PyQt5.QtWidgets import QDialog, QLabel, QComboBox, QPushButton
from PyQt5.QtCore import Qt

LOG = logging.getLogger('client')


class DelContactDialog(QDialog):
    """
    Dialogue for deleting a contact. Maintains the current contact list,
    does not have handlers for actions.
    """
    def __init__(self, database):
        super().__init__()
        self.database = database
        self.setFixedSize(350, 120)
        self.setWindowTitle('Choose friend to delete from list:')
        self.setAttribute(Qt.WA_DeleteOnClose)
        self.setModal(True)
        self.selector_label = QLabel('Choose friend to delete from list:', self)
        self.selector_label.setFixedSize(200, 20)
        self.selector_label.move(10, 0)
        self.selector = QComboBox(self)
        self.selector.setFixedSize(200, 20)
        self.selector.move(10, 30)
        self.btn_ok = QPushButton('Remove', self)
        self.btn_ok.setFixedSize(100, 30)
        self.btn_ok.move(230, 20)
        self.btn_cancel = QPushButton('Cancel', self)
        self.btn_cancel.setFixedSize(100, 30)
        self.btn_cancel.move(230, 60)
        self.btn_cancel.clicked.connect(self.close)
        self.selector.addItems(sorted(self.database.get_friends()))



from sqlalchemy import create_engine, Table, Column, Integer, String, Text, MetaData, DateTime
from sqlalchemy.orm import mapper, sessionmaker
import datetime
import logging

LOG = logging.getLogger('client')


class ClientDB:
    """
    The class is a wrapper for working with the client database.
    Uses SQLite database, implemented with
    QLAlchemy ORM and the classic approach is used.
    """
    class AllUsers:
        """
        All Users table class.
        """
        def __init__(self, user):
            self.id = None
            self.username = user

    class Friends:
        """
        Friends table class.
        """
        def __init__(self, contact):
            self.id = None
            self.name = contact

    class MessageHistory:
        """
        Message statistic table class.
        """
        def __init__(self, contact, direction, message):
            self.id = None
            self.contact = contact
            self.direction = direction
            self.message = message
            self.date = datetime.datetime.now()

    def __init__(self, username):
        self.database_engine = create_engine(f'sqlite:///dbs/client_{username}.db3',
                                             echo=False, pool_recycle=7200,
                                             connect_args={'check_same_thread': False})
        self.metadata = MetaData()
        users = Table('all_users', self.metadata,
                      Column('id', Integer, primary_key=True),
                      Column('username', String))

        friends = Table('friends', self.metadata,
                        Column('id', Integer, primary_key=True),
                        Column('name', String, unique=True))

        history = Table('message_history', self.metadata,
                        Column('id', Integer, primary_key=True),
                        Column('contact', String),
                        Column('direction', String),
                        Column('message', Text),
                        Column('date', DateTime))

        self.metadata.create_all(self.database_engine)
        mapper(self.AllUsers, users)
        mapper(self.Friends, friends)
        mapper(self.MessageHistory, history)
        connection = sessionmaker(bind=self.database_engine)
        self.session = connection()
        self.session.query(self.Friends).delete()
        self.session.commit()

    def add_friend_to_list(self, friend):
        """
        Add user to friends list method.
        :param friend:
        :return:
        """
        if not self.session.query(self.Friends).filter_by(name=friend).count():
            friend_row = self.Friends(friend)
            self.session.add(friend_row)
            self.session.commit()
            LOG.debug(f'We have a new friend: {friend}')

    def del_friend_from_list(self, friend):
        """
        Remove user from friends list method.
        :param friend:
        :return:
        """
        self.session.query(self.Friends).filter_by(name=friend).delete()
        LOG.debug(f'Deleting {friend} from friends list.')

    def add_all_users(self, users_list):
        """
        Fill All User table method.
        :param users_list:
        :return:
        """
        LOG.debug(f'Fill all users: {users_list}')
        self.session.query(self.AllUsers).delete()
        for user in users_list:
            user_row = self.AllUsers(user)
            self.session.add(user_row)
        self.session.commit()

    def save_message(self, from_user, direction, message):
        """
        Save message to client storage method.
        :param from_user:
        :param direction:
        :param message:
        :return:
        """
        LOG.debug(f'New message in history. From {from_user}.')
        message_row = self.MessageHistory(from_user, direction, message)
        self.session.add(message_row)
        self.session.commit()

    def get_friends(self):
        """
        Get full friends list method.
        :return:
        """
        LOG.debug('Getting friends list.')
        return [contact[0] for contact in self.session.query(self.Friends.name).all()]

    def get_all_users(self):
        """
        Get all users method.
        :return:
        """
        LOG.debug('Getting all users list.')
        return [user[0] for user in self.session.query(self.AllUsers.username).all()]

    def check_user(self, user):
        """
        Check user in all users table method.
        :param user:
        :return:
        """
        LOG.debug(f'Checking user in all users: {user}.')
        if self.session.query(self.AllUsers).filter_by(username=user).count():
            return True
        else:
            return False

    def check_friend(self, friend):
        """
        Check is user in a friends list method.
        :param friend:
        :return:
        """
        LOG.debug(f'Checking is user {friend} in friends list.')
        if self.session.query(self.Friends).filter_by(name=friend).count():
            return True
        else:
            return False

    def get_history(self, friend):
        """
        Get messages history with friend method.
        :param friend:
        :return:
        """
        query = self.session.query(self.MessageHistory).filter_by(contact=friend)
        return [(history_row.contact, history_row.direction, history_row.message, history_row.date)
                for history_row in query.all()]



import datetime
import sys
import logging

sys.path.append('../')

LOGGER = logging.getLogger('client')

def logger(log_func):
    """
    A decorator that logs function calls.
    Saves debug events containing
    information about the name of the called function, the parameters with which
    the function is called, and the module is calling the function.
    :param log_func:
    :return:
    """
    def log_saver(*args, **kwargs):
        result = log_func(*args, **kwargs)
        LOGGER.debug(f'{datetime.datetime.now()}. Function {log_func.__name__} is called from module {log_func.__module__}', stacklevel=2)
        return result
    return log_saver




from PyQt5.QtWidgets import QMainWindow, qApp, QMessageBox
from PyQt5.QtGui import QStandardItemModel, QStandardItem, QBrush, QColor
from PyQt5.QtCore import pyqtSlot, Qt
from Cryptodome.Cipher import PKCS1_OAEP
from Cryptodome.PublicKey import RSA
import base64
import sys
import json
import logging

from ServerAPP.common.variables import MESSAGE_TEXT, SENDER

sys.path.append('../../')

from ClientAPP.client.main_client_gui_conv import Ui_MainClientWindow
from ClientAPP.client.add_friend import AddContactDialog
from ClientAPP.client.del_friend import DelContactDialog

LOG = logging.getLogger('client')
CON_ERR = 'Lost connection!'
TIME_ERR = 'Timeout!'


class ClientMainWindow(QMainWindow):
    """
    Main user GUI class.
    Contains all the main logic of the client module.
    Window configuration created in QTDesigner and loaded from
    converted file main_client_gui_conv.py
    """
    def __init__(self, database, transport, keys):
        super().__init__()
        self.database = database
        self.transport = transport
        self.decrypter = PKCS1_OAEP.new(keys)
        self.ui = Ui_MainClientWindow()
        self.ui.setupUi(self)
        self.ui.menu_exit.triggered.connect(qApp.exit)
        self.ui.btn_send.clicked.connect(self.send_message)
        self.ui.btn_add_contact.clicked.connect(self.add_contact_window)
        self.ui.menu_add_contact.triggered.connect(self.add_contact_window)
        self.ui.btn_remove_contact.clicked.connect(self.delete_contact_window)
        self.ui.menu_del_contact.triggered.connect(self.delete_contact_window)
        self.contacts_model = None
        self.history_model = None
        self.messages = QMessageBox()
        self.current_chat = None
        self.current_chat_key = None
        self.encryptor = None
        self.ui.list_messages.setHorizontalScrollBarPolicy(Qt.ScrollBarAlwaysOff)
        self.ui.list_messages.setWordWrap(True)
        self.ui.list_contacts.doubleClicked.connect(self.select_active_user)
        self.clients_list_update()
        self.set_disabled_input()
        self.show()

    def set_disabled_input(self):
        """
        Deactivating fields method.
        :return:
        """
        self.ui.label_new_message.setText('Double click to choose destination user.')
        self.ui.text_message.clear()
        if self.history_model:
            self.history_model.clear()
        self.ui.btn_clear.setDisabled(True)
        self.ui.btn_send.setDisabled(True)
        self.ui.text_message.setDisabled(True)
        self.encryptor = None
        self.current_chat = None
        self.current_chat_key = None

    def history_list_update(self):
        """
        Fill history messaging method.
        :return:
        """
        dataset = sorted(self.database.get_history(self.current_chat), key=lambda item: item[3])
        if not self.history_model:
            self.history_model = QStandardItemModel()
            self.ui.list_messages.setModel(self.history_model)
        self.history_model.clear()
        length = len(dataset)
        start_index = 0
        if length > 20:
            start_index = length - 20
        for i in range(start_index, length):
            item = dataset[i]
            if item[1] == 'in':
                mess = QStandardItem(f'Incoming from {item[3].replace(microsecond=0)}:\n {item[2]}')
                mess.setEditable(False)
                mess.setBackground(QBrush(QColor(255, 213, 213)))
                mess.setTextAlignment(Qt.AlignLeft)
                self.history_model.appendRow(mess)
            else:
                mess = QStandardItem(f'Outgoing to {item[3].replace(microsecond=0)}:\n {item[2]}')
                mess.setEditable(False)
                mess.setTextAlignment(Qt.AlignRight)
                mess.setBackground(QBrush(QColor(204, 255, 204)))
                self.history_model.appendRow(mess)
        self.ui.list_messages.scrollToBottom()

    def select_active_user(self):
        """
        Double click event in friends list.
        :return:
        """
        self.current_chat = self.ui.list_contacts.currentIndex().data()
        self.set_active_user()

    def set_active_user(self):
        """
        Activate chat with friend method.
        :return:
        """
        try:
            self.current_chat_key = self.transport.key_request(
                self.current_chat)
            LOG.debug(f'Loaded public key for {self.current_chat}')
            if self.current_chat_key:
                self.encryptor = PKCS1_OAEP.new(
                    RSA.import_key(self.current_chat_key))
        except (OSError, json.JSONDecodeError):
            self.current_chat_key = None
            self.encryptor = None
            LOG.debug(f'Error loading public key for {self.current_chat}')
        if not self.current_chat_key:
            self.messages.warning(
                self, 'Error', 'Crypto key for user not found.')
            return
        self.ui.label_new_message.setText(
            f'Input message for user {self.current_chat}:')
        self.ui.btn_clear.setDisabled(False)
        self.ui.btn_send.setDisabled(False)
        self.ui.text_message.setDisabled(False)
        self.history_list_update()

    def clients_list_update(self):
        """
        Update friends list method.
        :return:
        """
        friends_list = self.database.get_friends()
        self.contacts_model = QStandardItemModel()
        for i in sorted(friends_list):
            item = QStandardItem(i)
            item.setEditable(False)
            self.contacts_model.appendRow(item)
        self.ui.list_contacts.setModel(self.contacts_model)

    def add_contact_window(self):
        """
        Create add friend window method.
        :return:
        """
        global select_dialog
        select_dialog = AddContactDialog(self.transport, self.database)
        select_dialog.btn_ok.clicked.connect(lambda: self.add_contact_action(select_dialog))
        select_dialog.show()

    def add_contact_action(self, item):
        """
        Adding user to friends list event method.
        :param item:
        :return:
        """
        new_contact = item.selector.currentText()
        self.add_contact(new_contact)
        item.close()

    def add_contact(self, new_contact):
        """
        Adding user to friends list in client and server DB method.
        :param new_contact:
        :return:
        """
        try:
            self.transport.add_friend(new_contact)
        except OSError as err:
            if err.errno:
                self.messages.critical(self, 'Error', CON_ERR)
                self.close()
            self.messages.critical(self, 'Error', TIME_ERR)
        else:
            self.database.add_friend_to_list(new_contact)
            new_contact = QStandardItem(new_contact)
            new_contact.setEditable(False)
            self.contacts_model.appendRow(new_contact)
            LOG.debug(f'Added friend {new_contact}')
            self.messages.information(self, 'Success', 'Friend added.')

    def delete_contact_window(self):
        """
        Remove user from friends list event method.
        :return:
        """
        global remove_dialog
        remove_dialog = DelContactDialog(self.database)
        remove_dialog.btn_ok.clicked.connect(lambda: self.delete_contact(remove_dialog))
        remove_dialog.show()

    def delete_contact(self, item):
        """
        Remove user from friends list in client and server DB method.
        :param item:
        :return:
        """
        selected = item.selector.currentText()
        try:
            self.transport.remove_friend(selected)
        except OSError as err:
            if err.errno:
                self.messages.critical(self, 'Error', CON_ERR)
                self.close()
            self.messages.critical(self, 'Error', TIME_ERR)
        else:
            self.database.del_friend_from_list(selected)
            self.clients_list_update()
            LOG.debug(f'Removed from friends list {selected}')
            self.messages.information(self, 'Success', 'User removed.')
            item.close()
            if selected == self.current_chat:
                self.current_chat = None
                self.set_disabled_input()

    def send_message(self):
        """
        Encrypt and send message to friend method.
        :return:
        """
        message_text = self.ui.text_message.toPlainText()
        self.ui.text_message.clear()
        if not message_text:
            return
        message_text_encrypted = self.encryptor.encrypt(message_text.encode('utf8'))
        message_text_encrypted_base64 = base64.b64encode(message_text_encrypted)
        try:
            self.transport.send_message(self.current_chat, message_text_encrypted_base64.decode('ascii'))
        except OSError as err:
            if err.errno:
                self.messages.critical(self, 'Error', CON_ERR)
                self.close()
            self.messages.critical(self, 'Error', TIME_ERR)
        else:
            self.database.save_message(self.current_chat, 'out', message_text)
            LOG.debug(f'Message was sent to {self.current_chat}: {message_text}')
            self.history_list_update()

    @pyqtSlot(dict)
    def message(self, message):
        """
        Slot handler of incoming messages, performs decryption
        received messages and their saving in the message history.
        Asks the user if the message is not from the current one
        interlocutor. Changes the interlocutor if necessary.
        :param message:
        :return:
        """
        encrypted_message = base64.b64decode(message[MESSAGE_TEXT])
        LOG.debug(f'Encrypted message {encrypted_message}')
        try:
            decrypted_message = self.decrypter.decrypt(encrypted_message)
            LOG.debug(f'Decrypted message {decrypted_message}')
        except (ValueError, TypeError):
            self.messages.warning(
                self, 'Error', 'Cannot decode message.')
            return
        sender = message[SENDER]
        self.database.save_message(sender, 'in', decrypted_message.decode('utf8'))
        if sender == self.current_chat:
            self.history_list_update()
        else:
            if self.database.check_friend(sender):
                if self.messages.question(self, 'New message', f'New message from {sender}, open chat?',
                                          QMessageBox.Yes, QMessageBox.No) == QMessageBox.Yes:
                    self.current_chat = sender
                    self.set_active_user()
            else:
                if self.messages.question(self, 'New message',
                                          f'New message from {sender}.\n User not in friends list.\n Add and open chat?',
                                          QMessageBox.Yes, QMessageBox.No) == QMessageBox.Yes:
                    self.add_contact(sender)
                    self.current_chat = sender
                    self.database.save_message(self.current_chat, 'in', decrypted_message.decode('utf8'))
                    self.set_active_user()

    @pyqtSlot()
    def connection_lost(self):
        """
        Slot handler of Connection lost.
        :return:
        """
        self.messages.warning(self, 'Connection error', CON_ERR)
        self.close()

    def make_connection(self, trans_obj):
        """
        Connect signals with slots.
        :param trans_obj:
        :return:
        """
        trans_obj.new_message.connect(self.message)
        trans_obj.connection_lost.connect(self.connection_lost)
        trans_obj.message_205.connect(self.sig_205)

    @pyqtSlot()
    def sig_205(self):
        """
        Database update from server method.
        :return:
        """
        if self.current_chat and not self.database.check_user(self.current_chat):
            self.messages.warning(self, 'Sorry', 'User deleted.')
            self.set_disabled_input()
            self.current_chat = None
        self.clients_list_update()



import binascii
import hashlib
import hmac
import json
import sys
import socket
import threading
import time
import logging
from PyQt5.QtCore import QObject, pyqtSignal

from ClientAPP.common.errors import ClientError

sys.path.append('../../')
from ClientAPP.common.decorators import logger
from ClientAPP.common.variables import ACTION, PRESENCE, TIME, USER, ACCOUNT_NAME, RESPONSE, RESP_OK, \
    RESP_BAD, EXIT, SENDER, DESTINATION, MESSAGE, MESSAGE_TEXT, GET_FRIENDS, LIST_INFO, REMOVE_FRIEND, ADD_FRIEND, \
    USERS_REQUEST, PUBLIC_KEY, RESP_511, DATA, ERROR, PUBLIC_KEY_REQUEST
from ClientAPP.common.utils import get_message, send_message

LOG = logging.getLogger('client')
sock_lock = threading.Lock()
CON_ERR = 'Connection to server lost.'


class ClientExchange(threading.Thread, QObject):
    """
    Exchange class. Responsible for interacting with the server.
    """
    new_message = pyqtSignal(dict)
    connection_lost = pyqtSignal()
    message_205 = pyqtSignal()

    def __init__(self, bind_port, bind_ip, database, username, passwd, keys):
        threading.Thread.__init__(self)
        QObject.__init__(self)
        self.database = database
        self.username = username
        self.keys = keys
        self.password = passwd
        self.exchange = None
        self.connection_init(bind_port, bind_ip)
        try:
            self.user_list_request()
            self.friends_list_request()
        except OSError as err:
            if err.errno:
                LOG.error(CON_ERR)
                raise ClientError(CON_ERR)
            LOG.error('Timeout connection.')
        except json.JSONDecodeError:
            LOG.error(CON_ERR)
            raise ClientError(CON_ERR)
        self.running = True

    def connection_init(self, bind_port, bind_ip):
        """
        Establishing connection to server method.
        :param bind_port:
        :param bind_ip:
        :return:
        """
        self.exchange = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.exchange.settimeout(5)
        connected = False
        for i in range(5):
            LOG.debug(f'Trying connect to server. Try {i + 1}.')
            try:
                self.exchange.connect((bind_ip, bind_port))
            except (OSError, ConnectionRefusedError):
                pass
            else:
                connected = True
                break
            time.sleep(1)
        if not connected:
            LOG.error('Cannot connect to te server.')
            raise ClientError('Cannot connect to te server.')
        LOG.debug('Connection established. Authentication.')

        b_pass = self.password.encode('utf-8')
        salt = self.username.lower().encode('utf-8')
        hash_pass = hashlib.pbkdf2_hmac('sha512', b_pass, salt, 10000)
        str_hash_pass = binascii.hexlify(hash_pass)
        LOG.debug(f'Password hash: {str_hash_pass}')
        pubkey = self.keys.publickey().export_key().decode('ascii')

        with sock_lock:
            presense = {
                ACTION: PRESENCE,
                TIME: time.time(),
                USER: {
                    ACCOUNT_NAME: self.username,
                    PUBLIC_KEY: pubkey
                }
            }
            try:
                send_message(self.exchange, presense)
                ans = get_message(self.exchange)
                LOG.debug(f'Server response = {ans}.')
                if RESPONSE in ans:
                    if ans[RESPONSE] == 400:
                        raise ClientError(ans[ERROR])
                    elif ans[RESPONSE] == 511:
                        ans_data = ans[DATA]
                        hash_data = hmac.new(str_hash_pass, ans_data.encode('utf-8'), 'MD5')
                        digest = hash_data.digest()
                        my_ans = RESP_511
                        my_ans[DATA] = binascii.b2a_base64(
                            digest).decode('ascii')
                        send_message(self.exchange, my_ans)
                        self.process_answer(get_message(self.exchange))
            except (OSError, json.JSONDecodeError) as err:
                LOG.debug(f'Connection error.', exc_info=err)
                raise ClientError('Connection error.')

    def process_answer(self, message):
        """
        Processing server response method.
        :param message:
        :return:
        """
        if RESPONSE in message:
            if message[RESPONSE] == 200:
                LOG.debug('Server return OK for PRESENCE message')
                return RESP_OK
            elif message[RESPONSE] == 400:
                LOG.error('Server return BAD for PRESENCE message')
                return RESP_BAD
            elif message[RESPONSE] == 205:
                self.user_list_request()
                self.friends_list_request()
                self.message_205.emit()
            else:
                LOG.error('ValueError in server response.')

        elif ACTION in message and message[ACTION] == MESSAGE and SENDER in message and DESTINATION in message \
                and MESSAGE_TEXT in message and message[DESTINATION] == self.username:
            LOG.info(f'Message from user {message[SENDER]}: {message[MESSAGE_TEXT]}')
            self.new_message.emit(message)

    @logger
    def create_presence(self):
        """
        Method to create presence (initial) message for server.
        :return:
        """
        out = {ACTION: PRESENCE, TIME: time.time(), USER: {ACCOUNT_NAME: self.username}}
        LOG.debug(f'Presence created from {self.username}.')
        return out

    @logger
    def friends_list_request(self):
        """
        Get friends list from server method.
        :return:
        """
        LOG.debug(f'Friends list request for {self.username}')
        req = {ACTION: GET_FRIENDS, TIME: time.time(), USER: self.username}
        with sock_lock:
            send_message(self.exchange, req)
            answer = get_message(self.exchange)
        LOG.debug(f'Response friends list:  {answer}')
        if RESPONSE in answer and answer[RESPONSE] == 202:
            for friend in answer[LIST_INFO]:
                self.database.add_friend_to_list(friend)
        else:
            raise ClientError('Bad response!')

    @logger
    def user_list_request(self):
        """
        Get all users list from server method.
        :return:
        """
        LOG.debug(f'Request all users list {self.username}')
        req = {ACTION: USERS_REQUEST, TIME: time.time(), ACCOUNT_NAME: self.username}
        with sock_lock:
            send_message(self.exchange, req)
            answer = get_message(self.exchange)
        if RESPONSE in answer and answer[RESPONSE] == 202:
            self.database.add_all_users(answer[LIST_INFO])
        else:
            raise ClientError('Error while request all users list!')

    def key_request(self, user):
        """
        Public key request method.
        :param user:
        :return:
        """
        LOG.debug(f'Publik key request for user {user}')
        req = {ACTION: PUBLIC_KEY_REQUEST, TIME: time.time(), ACCOUNT_NAME: user}
        with sock_lock:
            send_message(self.exchange, req)
            ans = get_message(self.exchange)
        if RESPONSE in ans and ans[RESPONSE] == 511:
            return ans[DATA]
        else:
            LOG.error(f'Error while requesting pubkey for user {user}.')

    @logger
    def add_friend(self, friend):
        """
        Add user to friends list request method.
        :param friend:
        :return:
        """
        LOG.debug(f'Adding user to friends list {friend}')
        req = {ACTION: ADD_FRIEND, TIME: time.time(), USER: self.username, ACCOUNT_NAME: friend}
        with sock_lock:
            send_message(self.exchange, req)
            self.process_answer(get_message(self.exchange))

    @logger
    def remove_friend(self, friend):
        """
        Remove user from friends list request method.
        :param friend:
        :return:
        """
        LOG.debug(f'Remove friend {friend} from list.')
        req = {ACTION: REMOVE_FRIEND, TIME: time.time(), USER: self.username, ACCOUNT_NAME: friend}
        with sock_lock:
            send_message(self.exchange, req)
            self.process_answer(get_message(self.exchange))

    @logger
    def exchange_stop(self):
        """
        Stopping exchange with server method.
        :return:
        """
        self.running = False
        message = {ACTION: EXIT, TIME: time.time(), ACCOUNT_NAME: self.username}
        with sock_lock:
            try:
                send_message(self.exchange, message)
            except OSError:
                pass
        LOG.debug('Exchange is stopping.')
        time.sleep(0.5)

    @logger
    def send_message(self, dest, message):
        """
        Send message to server method.
        :param dest:
        :param message:
        :return:
        """
        message_dict = {ACTION: MESSAGE, SENDER: self.username, DESTINATION: dest, TIME: time.time(),
                        MESSAGE_TEXT: message}
        LOG.debug(f'Created message dict: {message_dict}')
        with sock_lock:
            send_message(self.exchange, message_dict)
            self.process_answer(get_message(self.exchange))
            LOG.debug(f'Message was sent to {dest}.')

    def run(self):
        """
        Starting exchange with server method.
        :return:
        """
        while self.running:
            time.sleep(1)
            message = None
            with sock_lock:
                try:
                    self.exchange.settimeout(0.5)
                    message = get_message(self.exchange)
                except OSError as err:
                    if err.errno:
                        LOG.error('Lost connection to server.')
                        self.running = False
                        self.connection_lost.emit()
                except (json.JSONDecodeError, TypeError):
                    LOG.error('Lost connection to server.')
                    self.running = False
                    self.connection_lost.emit()
                finally:
                    self.exchange.settimeout(5)
            if message:
                LOG.debug(f'Message from server: {message}')
                self.process_answer(message)



import sys
import json
from ClientAPP.common.variables import MAX_PKG, ENCODING
from ClientAPP.common.decorators import logger
sys.path.append('../')


@logger
def get_message(client):
    """
    Get message from socket function.
    :param client:
    :return:
    """
    encoded_response = client.recv(MAX_PKG)
    json_response = encoded_response.decode(ENCODING)
    response = json.loads(json_response)
    if isinstance(response, dict):
        return response
    else:
        raise TypeError


@logger
def send_message(sock, message):
    """
    Send message for socket function.
    :param sock:
    :param message:
    :return:
    """
    json_req = json.dumps(message)
    package_to_send = json_req.encode(ENCODING)
    sock.send(package_to_send)



import argparse
import os
import sys
from PyQt5.QtWidgets import QApplication, QMessageBox
import logging
from ClientAPP.common.decorators import logger
from ClientAPP.common.variables import DEF_IP, DEF_PORT
from ClientAPP.client.main_client_gui import ClientMainWindow
from ClientAPP.client.uname_dialog import UserNameDialog
from ClientAPP.client.exchange_proto import ClientExchange
from ClientAPP.client.client_storage import ClientDB
from Cryptodome.PublicKey import RSA

LOG = logging.getLogger('client')


@logger
def arg_parser():
    parser = argparse.ArgumentParser()
    parser.add_argument('addr', default=DEF_IP, nargs='?')
    parser.add_argument('port', default=DEF_PORT, type=int, nargs='?')
    parser.add_argument('-n', '--name', default=None, nargs='?')
    parser.add_argument('-p', '--password', default='', nargs='?')
    arguments = parser.parse_args(sys.argv[1:])
    server_address = arguments.addr
    server_port = arguments.port
    client_name = arguments.name
    client_passwd = arguments.password
    if not 1023 < server_port < 65536:
        LOG.critical(f'Port not valid: {server_port}. Valid values: 1024 - 65535.')
        sys.exit(1)
    return server_address, server_port, client_name, client_passwd


def start_client():
    bind_ip, bind_port, client_name, client_passwd = arg_parser()
    client_app = QApplication(sys.argv)
    start_dialog = UserNameDialog()
    if not client_name or not client_passwd:
        client_app.exec_()
        if start_dialog.ok_pressed:
            client_name = start_dialog.client_name.text()
            client_passwd = start_dialog.client_passwd.text()
            LOG.debug(f'Using USERNAME = {client_name}, PASSWD = {client_passwd}.')
        else:
            exit(0)
    LOG.info(f'Starting client with username: {client_name}. Connecting to server {bind_ip}:{bind_port}')

    dir_path = os.path.dirname(os.path.realpath(__file__))
    key_file = os.path.join(dir_path, 'keys', f'{client_name}.key')
    if not os.path.exists(key_file):
        keys = RSA.generate(2048, os.urandom)
        with open(key_file, 'wb') as key:
            key.write(keys.export_key())
    else:
        with open(key_file, 'rb') as key:
            keys = RSA.import_key(key.read())
    LOG.debug("Keys successfully loaded.")

    database = ClientDB(client_name)
    try:
        exchange = ClientExchange(bind_port, bind_ip, database, client_name, client_passwd, keys)
    except (OSError, ValueError) as error:
        message = QMessageBox()
        message.critical(start_dialog, 'Server error', error.text)
        exit(1)
    exchange.setDaemon(True)
    exchange.start()
    del start_dialog

    main_window = ClientMainWindow(database, exchange,  keys)
    main_window.make_connection(exchange)
    main_window.setWindowTitle(f'Python-messenger. Client - {client_name}')
    client_app.exec_()
    exchange.exchange_stop()
    exchange.join()


if __name__ == '__main__':
    start_client()




from setuptools import setup, find_packages

setup(name = "PthMsn-Client-v1",
      version = "1.0",
      description = "Python Messenger Client. Version 1.",
      author = "NukeDancer",
      author_email = "nukedancer@yandex.ru",
      packages=find_packages(),      install_requires=['PyQt5', 'sqlalchemy', 'pycryptodome', 'pycryptodomex']
      )



import sys

from PyQt5.QtWidgets import QDialog, QPushButton, QLineEdit, QLabel, qApp
sys.path.append('../../')


class UserNameDialog(QDialog):
    """
    User login and password dialogue class.
    """
    def __init__(self):
        super().__init__()
        self.ok_pressed = False
        self.setWindowTitle('Starting...')
        self.setFixedSize(175, 135)
        self.label = QLabel('Input your username:', self)
        self.label.move(10, 10)
        self.label.setFixedSize(150, 10)
        self.client_name = QLineEdit(self)
        self.client_name.setFixedSize(154, 20)
        self.client_name.move(10, 30)
        self.btn_ok = QPushButton('Start', self)
        self.btn_ok.move(10, 105)
        self.btn_ok.clicked.connect(self.click)
        self.btn_cancel = QPushButton('Exit', self)
        self.btn_cancel.move(90, 105)
        self.btn_cancel.clicked.connect(qApp.exit)
        self.label_passwd = QLabel('Enter password:', self)
        self.label_passwd.move(10, 55)
        self.label_passwd.setFixedSize(150, 15)
        self.client_passwd = QLineEdit(self)
        self.client_passwd.setFixedSize(154, 20)
        self.client_passwd.move(10, 75)
        self.client_passwd.setEchoMode(QLineEdit.Password)
        self.show()

    def click(self):
        """
        OK click handler method.
        :return:
        """
        if self.client_name.text() and self.client_passwd.text():
            self.ok_pressed = True
            qApp.exit()



import dis


class ClientMetaVerify(type):
    """
    Metaclass for Client Verifying.
    """
    def __init__(self, clsname, bases, clsdict):
        mtdlist = []
        for func in clsdict:
            try:
                func_out = dis.get_instructions(clsdict[func])
            except TypeError:
                pass
            else:
                for i in func_out:
                    if i.opname == 'LOAD_GLOBAL' and i.argval not in mtdlist:
                        mtdlist.append(i.argval)
        for command in ('accept', 'listen', 'socket'):
            if command in mtdlist:
                raise TypeError('Privat method is not allowed.')
        if 'get_message' not in mtdlist or 'send_message' not in mtdlist:
            raise TypeError('Need socket functions.')
        super().__init__(clsname, bases, clsdict)



import logging


LOG_FILE = 'ClientAPP/log/client_logs/client.log'
LOG = logging.getLogger('client')
LOG_FORMAT = logging.Formatter('%(asctime)-s %(levelname)-s %(module)-s %(message)s')
FILE_HANDLER = logging.FileHandler(LOG_FILE, encoding='utf-8')
FILE_HANDLER.setLevel(logging.DEBUG)
FILE_HANDLER.setFormatter(LOG_FORMAT)
LOG.addHandler(FILE_HANDLER)
LOG.setLevel(logging.DEBUG)
